@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2019 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -14,7 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef WEECHAT_STRING_H
@@ -39,6 +39,8 @@ struct t_hashtable;
 extern char *string_strndup (const char *string, int length);
 extern char *string_cut (const char *string, int length, int count_suffix,
                          int screen, const char *cut_suffix);
+extern char *string_reverse (const char *string);
+extern char *string_repeat (const char *string, int count);
 extern void string_tolower (char *string);
 extern void string_toupper (char *string);
 extern int string_strcasecmp (const char *string1, const char *string2);
@@ -55,6 +57,8 @@ extern int string_strcmp_ignore_chars (const char *string1,
 extern const char *string_strcasestr (const char *string, const char *search);
 extern int string_match (const char *string, const char *mask,
                          int case_sensitive);
+extern int string_match_list (const char *string, const char **masks,
+                              int case_sensitive);
 extern char *string_replace (const char *string, const char *search,
                              const char *replace);
 extern char *string_expand_home (const char *path);
@@ -83,10 +87,11 @@ extern char *string_replace_regex (const char *string, void *regex,
                                    char *(*callback)(void *data, const char *text),
                                    void *callback_data);
 extern char **string_split (const char *string, const char *separators,
-                            int keep_eol, int num_items_max, int *num_items);
+                            const char *strip_items, int flags,
+                            int num_items_max, int *num_items);
 extern char **string_split_shared (const char *string, const char *separators,
-                                   int keep_eol, int num_items_max,
-                                   int *num_items);
+                                   const char *strip_items, int flags,
+                                   int num_items_max, int *num_items);
 extern char **string_split_shell (const char *string, int *num_items);
 extern void string_free_split (char **split_string);
 extern void string_free_split_shared (char **split_string);
@@ -94,6 +99,8 @@ extern char *string_build_with_split_string (const char **split_string,
                                              const char *separator);
 extern char **string_split_command (const char *command, char separator);
 extern void string_free_split_command (char **split_command);
+extern char ***string_split_tags (const char *tags, int *num_tags);
+extern void string_free_split_tags (char ***split_tags);
 extern char *string_iconv (int from_utf8, const char *from_code,
                            const char *to_code, const char *string);
 extern char *string_iconv_to_internal (const char *charset, const char *string);
@@ -101,10 +108,12 @@ extern char *string_iconv_from_internal (const char *charset,
                                          const char *string);
 extern int string_fprintf (FILE *file, const char *data, ...);
 extern char *string_format_size (unsigned long long size);
-extern void string_encode_base16 (const char *from, int length, char *to);
-extern int string_decode_base16 (const char *from, char *to);
-extern void string_encode_base64 (const char *from, int length, char *to);
-extern int string_decode_base64 (const char *from, char *to);
+extern int string_base16_encode (const char *from, int length, char *to);
+extern int string_base16_decode (const char *from, char *to);
+extern int string_base32_encode (const char *from, int length, char *to);
+extern int string_base32_decode (const char *from, char *to);
+extern int string_base64_encode (const char *from, int length, char *to);
+extern int string_base64_decode (const char *from, char *to);
 extern char *string_hex_dump (const char *data, int data_size,
                               int bytes_per_line,
                               const char *prefix, const char *suffix);
