@@ -1,7 +1,7 @@
 /*
  * gui-curses-color.c - color functions for Curses GUI
  *
- * Copyright (C) 2003-2014 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2019 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -285,6 +285,16 @@ gui_color_get_term_colors ()
     return gui_color_term_colors;
 }
 
+/*
+ * Gets number of color pairs supported by terminal.
+ */
+
+int
+gui_color_get_term_color_pairs ()
+{
+    return gui_color_term_color_pairs;
+}
+
 /*
  * Gets current pairs as arrays (one array for foregrounds, another for
  * backgrounds).
@@ -351,9 +361,11 @@ error:
  */
 
 int
-gui_color_timer_warning_pairs_full (void *data, int remaining_calls)
+gui_color_timer_warning_pairs_full (const void *pointer, void *data,
+                                    int remaining_calls)
 {
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
     (void) remaining_calls;
 
@@ -402,7 +414,7 @@ gui_color_get_pair (int fg, int bg)
             {
                 /* display warning if auto reset of pairs is disabled */
                 hook_timer (NULL, 1, 0, 1,
-                            &gui_color_timer_warning_pairs_full, NULL);
+                            &gui_color_timer_warning_pairs_full, NULL, NULL);
                 gui_color_warning_pairs_full = 1;
             }
             return 1;
@@ -578,8 +590,9 @@ gui_color_init_vars ()
         gui_color_term_color_pairs = COLOR_PAIRS;
         gui_color_term_can_change_color = (can_change_color ()) ? 1 : 0;
 
-        gui_color_num_pairs = (gui_color_term_color_pairs >= 256) ?
-            255 : gui_color_term_color_pairs - 1;
+        /* TODO: ncurses may support 65536, but short type used for pairs supports only 32768? */
+        gui_color_num_pairs = (gui_color_term_color_pairs >= 32768) ?
+            32767 : gui_color_term_color_pairs - 1;
         size = (gui_color_term_colors + 2)
             * (gui_color_term_colors + 2)
             * sizeof (gui_color_pairs[0]);
@@ -697,7 +710,7 @@ gui_color_init_pairs_weechat ()
 void
 gui_color_display_terminal_colors ()
 {
-    int lines, line, col, color;
+    int lines, columns, line, col, color;
     char str_line[1024], str_color[64];
 
     initscr ();
@@ -725,13 +738,14 @@ gui_color_display_terminal_colors ()
         printf ("%s\n", _("Default colors:"));
         printf ("------------------------------------------------------------"
                 "--------------------\n");
-        lines = (gui_color_term_colors < 16) ? gui_color_term_colors : 16;
+        columns = 16;
+        lines = (gui_color_term_colors - 1) / columns + 1;
         for (line = 0; line < lines; line++)
         {
             str_line[0] = '\0';
-            for (col = 0; col < 16; col++)
+            for (col = 0; col < columns; col++)
             {
-                color = (col * 16) + line;
+                color = line * columns + col;
                 if (color < gui_color_term_colors)
                 {
                     snprintf (str_color, sizeof (str_color),
@@ -790,7 +804,7 @@ gui_color_info_term_colors (char *buffer, int size)
 void
 gui_color_buffer_display ()
 {
-    int y, i, lines, line, col, color, max_color, num_items;
+    int y, i, lines, columns, line, col, color, max_color, num_items;
     char str_line[1024], str_color[64], str_rgb[64], **items;
     struct t_gui_color_palette *color_palette;
 
@@ -827,17 +841,18 @@ gui_color_buffer_display ()
                            gui_color_pairs_used,
                            gui_color_num_pairs - gui_color_pairs_used);
     }
+    columns = 16;
     max_color = (gui_color_use_term_colors) ?
-        gui_color_term_colors - 1 : gui_color_num_pairs;
-    if (max_color > 255)
-        max_color = 255;
-    lines = (max_color <= 64) ? 8 : 16;
+        gui_color_term_colors - 1 : gui_color_pairs_used;
+    /* round up to nearest multiple of columns */
+    max_color = (max_color / columns) * columns + columns - 1;
+    lines = max_color / columns + 1;
     for (line = 0; line < lines; line++)
     {
         str_line[0] = '\0';
-        for (col = 0; col < 16; col++)
+        for (col = 0; col < columns; col++)
         {
-            color = (col * lines) + line;
+            color = line * columns + col;
             if (color <= max_color)
             {
                 if (color == 0)
@@ -849,13 +864,11 @@ gui_color_buffer_display ()
                          || (color <= gui_color_pairs_used))
                 {
                     snprintf (str_color, sizeof (str_color),
-                              "%c%c%05d%c%03d%c",
+                              (color <= 999) ? "%c%c%05d %03d " : "%c%c%05d%5d",
                               GUI_COLOR_COLOR_CHAR,
                               GUI_COLOR_EXTENDED_CHAR,
                               color,
-                              (color == 0) ? '<' : ' ',
-                              color,
-                              (color == 0) ? '>' : ' ');
+                              color);
                 }
                 else
                 {
@@ -933,7 +946,13 @@ gui_color_buffer_display ()
         gui_chat_printf_y (gui_color_buffer, y++,
                            _("Nick colors:"));
         items = string_split (CONFIG_STRING(config_color_chat_nick_colors),
-                              ",", 0, 0, &num_items);
+                              ",",
+                              NULL,
+                              WEECHAT_STRING_SPLIT_STRIP_LEFT
+                              | WEECHAT_STRING_SPLIT_STRIP_RIGHT
+                              | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS,
+                              0,
+                              &num_items);
         if (items)
         {
             str_line[0] = '\0';
@@ -1034,9 +1053,10 @@ gui_color_buffer_display ()
  */
 
 int
-gui_color_timer_cb (void *data, int remaining_calls)
+gui_color_timer_cb (const void *pointer, void *data, int remaining_calls)
 {
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
     (void) remaining_calls;
 
@@ -1101,7 +1121,7 @@ gui_color_switch_colors ()
     if (gui_color_use_term_colors)
     {
         gui_color_hook_timer = hook_timer (NULL, 1000, 0, 0,
-                                           &gui_color_timer_cb, NULL);
+                                           &gui_color_timer_cb, NULL, NULL);
     }
 }
 
@@ -1132,10 +1152,12 @@ gui_color_reset_pairs ()
  */
 
 int
-gui_color_buffer_input_cb (void *data, struct t_gui_buffer *buffer,
+gui_color_buffer_input_cb (const void *pointer, void *data,
+                           struct t_gui_buffer *buffer,
                            const char *input_data)
 {
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
 
     if (string_strcasecmp (input_data, "e") == 0)
@@ -1164,9 +1186,11 @@ gui_color_buffer_input_cb (void *data, struct t_gui_buffer *buffer,
  */
 
 int
-gui_color_buffer_close_cb (void *data, struct t_gui_buffer *buffer)
+gui_color_buffer_close_cb (const void *pointer, void *data,
+                           struct t_gui_buffer *buffer)
 {
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
     (void) buffer;
 
@@ -1184,7 +1208,8 @@ gui_color_buffer_assign ()
 {
     if (!gui_color_buffer)
     {
-        gui_color_buffer = gui_buffer_search_by_name (NULL, "color");
+        gui_color_buffer = gui_buffer_search_by_name (NULL,
+                                                      GUI_COLOR_BUFFER_NAME);
         if (gui_color_buffer)
         {
             gui_color_buffer->input_callback = &gui_color_buffer_input_cb;
@@ -1202,13 +1227,14 @@ gui_color_buffer_open ()
 {
     if (!gui_color_buffer)
     {
-        gui_color_buffer = gui_buffer_new (NULL, "color",
-                                           &gui_color_buffer_input_cb, NULL,
-                                           &gui_color_buffer_close_cb, NULL);
+        gui_color_buffer = gui_buffer_new (
+            NULL, GUI_COLOR_BUFFER_NAME,
+            &gui_color_buffer_input_cb, NULL, NULL,
+            &gui_color_buffer_close_cb, NULL, NULL);
         if (gui_color_buffer)
         {
             if (!gui_color_buffer->short_name)
-                gui_color_buffer->short_name = strdup ("color");
+                gui_color_buffer->short_name = strdup (GUI_COLOR_BUFFER_NAME);
             gui_buffer_set (gui_color_buffer, "type", "free");
             gui_buffer_set (gui_color_buffer, "localvar_set_no_log", "1");
             gui_buffer_set (gui_color_buffer, "key_bind_meta-c", "/color switch");
@@ -1331,7 +1357,11 @@ gui_color_palette_new (int number, const char *value)
         str_alias = NULL;
         str_rgb = NULL;
 
-        items = string_split (value, ";", 0, 0, &num_items);
+        items = string_split (value, ";", NULL,
+                              WEECHAT_STRING_SPLIT_STRIP_LEFT
+                              | WEECHAT_STRING_SPLIT_STRIP_RIGHT
+                              | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS,
+                              0, &num_items);
         if (items)
         {
             for (i = 0; i < num_items; i++)
@@ -1461,6 +1491,7 @@ gui_color_init_weechat ()
     gui_color_build (GUI_COLOR_CHAT_NICK_SUFFIX, CONFIG_COLOR(config_color_chat_nick_suffix), CONFIG_COLOR(config_color_chat_bg));
     gui_color_build (GUI_COLOR_EMPHASIS, CONFIG_COLOR(config_color_emphasized), CONFIG_COLOR(config_color_emphasized_bg));
     gui_color_build (GUI_COLOR_CHAT_DAY_CHANGE, CONFIG_COLOR(config_color_chat_day_change), CONFIG_COLOR(config_color_chat_bg));
+    gui_color_build (GUI_COLOR_CHAT_VALUE_NULL, CONFIG_COLOR(config_color_chat_value_null), CONFIG_COLOR(config_color_chat_bg));
 
     /*
      * define old nick colors for compatibility on /upgrade with previous
@@ -1487,7 +1518,7 @@ gui_color_init_weechat ()
 void
 gui_color_alloc ()
 {
-    if (has_colors())
+    if (has_colors ())
     {
         start_color ();
         use_default_colors ();
