@@ -1,7 +1,7 @@
 /*
  * gui-curses-window.c - window display functions for Curses GUI
  *
- * Copyright (C) 2003-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2019 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -193,7 +193,13 @@ gui_window_clear (WINDOW *window, int fg, int bg)
     else
         bg = gui_weechat_colors[bg & GUI_COLOR_EXTENDED_MASK].background;
 
+#ifdef NCURSES_EXT_COLORS
+    cchar_t c;
+    setcchar (&c, L" ", attrs, gui_color_get_pair (fg, bg), NULL);
+    wbkgrndset (window, &c);
+#else
     wbkgdset (window, ' ' | COLOR_PAIR (gui_color_get_pair (fg, bg)) | attrs);
+#endif
     werase (window);
     wmove (window, 0, 0);
 }
@@ -205,9 +211,16 @@ gui_window_clear (WINDOW *window, int fg, int bg)
 void
 gui_window_clrtoeol (WINDOW *window)
 {
+#ifdef NCURSES_EXT_COLORS
+    cchar_t c;
+    setcchar (&c, L" ", A_NORMAL, gui_color_get_pair (gui_window_current_style_fg,
+                                                      gui_window_current_style_bg), NULL);
+    wbkgrndset (window, &c);
+#else
     wbkgdset (window,
               ' ' | COLOR_PAIR (gui_color_get_pair (gui_window_current_style_fg,
                                                     gui_window_current_style_bg)));
+#endif
     wclrtoeol (window);
 }
 
@@ -282,9 +295,8 @@ gui_window_reset_style (WINDOW *window, int weechat_color)
     gui_window_current_style_bg = -1;
     gui_window_current_color_attr = 0;
 
-    wattroff (window, A_ALL_ATTR);
-    wattron (window, COLOR_PAIR(gui_color_weechat_get_pair (weechat_color)) |
-             gui_color[weechat_color]->attributes);
+    wattr_set (window, gui_color[weechat_color]->attributes,
+               gui_color_weechat_get_pair (weechat_color), NULL);
 }
 
 /*
@@ -297,8 +309,8 @@ gui_window_reset_color (WINDOW *window, int weechat_color)
     gui_window_current_style_fg = gui_color[weechat_color]->foreground;
     gui_window_current_style_bg = gui_color[weechat_color]->background;
 
-    wattron (window, COLOR_PAIR(gui_color_weechat_get_pair (weechat_color)) |
-             gui_color[weechat_color]->attributes);
+    wattron (window, gui_color[weechat_color]->attributes);
+    wcolor_set (window, gui_color_weechat_get_pair (weechat_color), NULL);
 }
 
 /*
@@ -333,7 +345,7 @@ gui_window_set_color (WINDOW *window, int fg, int bg)
     gui_window_current_style_fg = fg;
     gui_window_current_style_bg = bg;
 
-    wattron (window, COLOR_PAIR(gui_color_get_pair (fg, bg)));
+    wcolor_set (window, gui_color_get_pair (fg, bg), NULL);
 }
 
 /*
@@ -532,7 +544,7 @@ gui_window_set_custom_color_pair (WINDOW *window, int pair)
     if ((pair >= 0) && (pair <= gui_color_num_pairs))
     {
         gui_window_remove_color_style (window, A_ALL_ATTR);
-        wattron (window, COLOR_PAIR(pair));
+        wcolor_set (window, pair, NULL);
     }
 }
 
@@ -792,7 +804,12 @@ gui_window_string_apply_color_fg_bg (unsigned char **string, WINDOW *window)
             ptr_string += 2;
         }
     }
-    if (ptr_string[0] == ',')
+    /*
+     * note: the comma is an old separator not used any more
+     * (since WeeChat 2.6), but we still use it here so in case of /upgrade
+     * this will not break colors in old messages
+     */
+    if ((ptr_string[0] == ',') || (ptr_string[0] == '~'))
     {
         ptr_string++;
         if (ptr_string[0] == GUI_COLOR_EXTENDED_CHAR)
@@ -1964,40 +1981,39 @@ gui_window_merge (struct t_gui_window *window)
         return 0;
 
     parent = window->ptr_tree->parent_node;
-    if (parent)
+    if (!parent)
+        return 0;
+
+    sister = (parent->child1->window == window) ?
+        parent->child2 : parent->child1;
+    if (!(sister->window))
+        return 0;
+
+    if (window->win_y == sister->window->win_y)
     {
-        sister = (parent->child1->window == window) ?
-            parent->child2 : parent->child1;
-
-        if (!(sister->window))
-            return 0;
-
-        if (window->win_y == sister->window->win_y)
-        {
-            /* horizontal merge */
-            separator = (CONFIG_BOOLEAN(config_look_window_separator_horizontal)) ? 1 : 0;
-            window->win_width += sister->window->win_width + separator;
-            window->win_width_pct += sister->window->win_width_pct;
-        }
-        else
-        {
-            /* vertical merge */
-            separator = (CONFIG_BOOLEAN(config_look_window_separator_vertical)) ? 1 : 0;
-            window->win_height += sister->window->win_height + separator;
-            window->win_height_pct += sister->window->win_height_pct;
-        }
-        if (sister->window->win_x < window->win_x)
-            window->win_x = sister->window->win_x;
-        if (sister->window->win_y < window->win_y)
-            window->win_y = sister->window->win_y;
-
-        gui_window_free (sister->window);
-        gui_window_tree_node_to_leaf (parent, window);
-
-        gui_window_switch_to_buffer (window, window->buffer, 1);
-        return 1;
+        /* horizontal merge */
+        separator = (CONFIG_BOOLEAN(config_look_window_separator_horizontal)) ? 1 : 0;
+        window->win_width += sister->window->win_width + separator;
+        window->win_width_pct += sister->window->win_width_pct;
     }
-    return 0;
+    else
+    {
+        /* vertical merge */
+        separator = (CONFIG_BOOLEAN(config_look_window_separator_vertical)) ? 1 : 0;
+        window->win_height += sister->window->win_height + separator;
+        window->win_height_pct += sister->window->win_height_pct;
+    }
+    if (sister->window->win_x < window->win_x)
+        window->win_x = sister->window->win_x;
+    if (sister->window->win_y < window->win_y)
+        window->win_y = sister->window->win_y;
+
+    gui_window_free (sister->window);
+    gui_window_tree_node_to_leaf (parent, window);
+
+    gui_window_switch_to_buffer (window, window->buffer, 1);
+
+    return 1;
 }
 
 /*
@@ -2042,6 +2058,34 @@ gui_window_merge_all (struct t_gui_window *window)
     }
 }
 
+/*
+ * Closes a window.
+ *
+ * Returns:
+ *   1: OK
+ *   0: error
+ */
+
+int
+gui_window_close (struct t_gui_window *window)
+{
+    struct t_gui_window_tree *parent, *sister;
+
+    if (!gui_init_ok)
+        return 0;
+
+    parent = window->ptr_tree->parent_node;
+    if (!parent)
+        return 0;
+
+    sister = (parent->child1->window == window) ?
+        parent->child2 : parent->child1;
+    if (!(sister->window))
+        return 0;
+
+    return gui_window_merge (sister->window);
+}
+
 /*
  * Returns a code about position of 2 windows:
  *   0 = they're not side by side
@@ -2509,11 +2553,13 @@ gui_window_send_clipboard (const char *storage_unit, const char *text)
     text_base64 = malloc ((length * 4) + 1);
     if (text_base64)
     {
-        string_encode_base64 (text, length, text_base64);
-        fprintf (stderr, "\033]52;%s;%s\a",
-                 (storage_unit) ? storage_unit : "",
-                 text_base64);
-        fflush (stderr);
+        if (string_base64_encode (text, length, text_base64) >= 0)
+        {
+            fprintf (stderr, "\033]52;%s;%s\a",
+                     (storage_unit) ? storage_unit : "",
+                     text_base64);
+            fflush (stderr);
+        }
         free (text_base64);
     }
 }
@@ -2565,7 +2611,7 @@ gui_window_term_display_infos ()
     gui_chat_printf (NULL, "");
     gui_chat_printf (NULL, _("Terminal infos:"));
     gui_chat_printf (NULL, _("  TERM='%s', size: %dx%d"),
-                     getenv("TERM"), gui_term_cols, gui_term_lines);
+                     getenv ("TERM"), gui_term_cols, gui_term_lines);
 }
 
 /*
