@@ -1,7 +1,7 @@
 /*
  * gui-bar-window.c - bar window functions (used by all GUI)
  *
- * Copyright (C) 2003-2014 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2019 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -30,6 +30,7 @@
 
 #include "../core/weechat.h"
 #include "../core/wee-config.h"
+#include "../core/wee-hashtable.h"
 #include "../core/wee-hdata.h"
 #include "../core/wee-infolist.h"
 #include "../core/wee-log.h"
@@ -96,6 +97,9 @@ gui_bar_window_search_bar (struct t_gui_window *window, struct t_gui_bar *bar)
 {
     struct t_gui_bar_window *ptr_bar_win;
 
+    if (!window)
+        return NULL;
+
     for (ptr_bar_win = window->bar_windows; ptr_bar_win;
          ptr_bar_win = ptr_bar_win->next_bar_window)
     {
@@ -283,6 +287,10 @@ gui_bar_window_get_size (struct t_gui_bar *bar, struct t_gui_window *window,
     int total_size;
 
     total_size = 0;
+
+    if (!window)
+        return total_size;
+
     for (ptr_bar_window = window->bar_windows; ptr_bar_window;
          ptr_bar_window = ptr_bar_window->next_bar_window)
     {
@@ -313,6 +321,7 @@ gui_bar_window_get_size (struct t_gui_bar *bar, struct t_gui_window *window,
             }
         }
     }
+
     return total_size;
 }
 
@@ -327,8 +336,11 @@ gui_bar_window_calculate_pos_size (struct t_gui_bar_window *bar_window,
     int x1, y1, x2, y2;
     int add_bottom, add_top, add_left, add_right;
 
-    if (CONFIG_BOOLEAN(bar_window->bar->options[GUI_BAR_OPTION_HIDDEN]))
+    if (!bar_window
+        || CONFIG_BOOLEAN(bar_window->bar->options[GUI_BAR_OPTION_HIDDEN]))
+    {
         return;
+    }
 
     if (window)
     {
@@ -405,6 +417,9 @@ gui_bar_window_find_pos (struct t_gui_bar *bar, struct t_gui_window *window)
 {
     struct t_gui_bar_window *ptr_bar_window;
 
+    if (!window)
+        return NULL;
+
     for (ptr_bar_window = window->bar_windows; ptr_bar_window;
          ptr_bar_window = ptr_bar_window->next_bar_window)
     {
@@ -426,6 +441,9 @@ gui_bar_window_content_alloc (struct t_gui_bar_window *bar_window)
 {
     int i, j;
 
+    if (!bar_window)
+        return;
+
     bar_window->items_count = bar_window->bar->items_count;
     bar_window->items_subcount = NULL;
     bar_window->items_content = NULL;
@@ -535,6 +553,9 @@ gui_bar_window_content_free (struct t_gui_bar_window *bar_window)
 {
     int i, j;
 
+    if (!bar_window)
+        return;
+
     if (bar_window->items_content)
     {
         for (i = 0; i < bar_window->items_count; i++)
@@ -571,6 +592,9 @@ gui_bar_window_content_build_item (struct t_gui_bar_window *bar_window,
                                    struct t_gui_window *window,
                                    int index_item, int index_subitem)
 {
+    if (!bar_window)
+        return;
+
     if (bar_window->items_content)
     {
         if (bar_window->items_content[index_item][index_subitem])
@@ -605,6 +629,9 @@ gui_bar_window_content_build (struct t_gui_bar_window *bar_window,
 {
     int i, j;
 
+    if (!bar_window)
+        return;
+
     gui_bar_window_content_free (bar_window);
     gui_bar_window_content_alloc (bar_window);
 
@@ -621,11 +648,14 @@ gui_bar_window_content_build (struct t_gui_bar_window *bar_window,
  * Gets item or subitem content (first rebuilds content if refresh is needed).
  */
 
-char *
+const char *
 gui_bar_window_content_get (struct t_gui_bar_window *bar_window,
                             struct t_gui_window *window,
                             int index_item, int index_subitem)
 {
+    if (!bar_window)
+        return NULL;
+
     /* rebuild content if refresh is needed */
     if (bar_window->items_refresh_needed[index_item][index_subitem])
     {
@@ -638,8 +668,10 @@ gui_bar_window_content_get (struct t_gui_bar_window *bar_window,
 }
 
 /*
- * Gets content of a bar window, formatted for display, according to filling for
- * bar position.
+ * Gets content of a bar window, formatted for display, according to filling
+ * for bar position.
+ *
+ * Note: result must be freed after use.
  */
 
 char *
@@ -647,7 +679,8 @@ gui_bar_window_content_get_with_filling (struct t_gui_bar_window *bar_window,
                                          struct t_gui_window *window)
 {
     enum t_gui_bar_filling filling;
-    char *ptr_content, *content, *content2, str_reinit_color[32];
+    const char *ptr_content;
+    char *content, *content2, str_reinit_color[32];
     char str_reinit_color_space[32], str_reinit_color_space_start_line[32];
     char str_start_item[32];
     char *item_value, *item_value2, ****split_items, **linear_items;
@@ -657,9 +690,12 @@ gui_bar_window_content_get_with_filling (struct t_gui_bar_window *bar_window,
     int max_length, max_length_screen;
     int total_items, columns, lines;
 
-    if (!bar_window->items_subcount || !bar_window->items_content
+    if (!bar_window
+        || !bar_window->items_subcount || !bar_window->items_content
         || !bar_window->items_num_lines || !bar_window->items_refresh_needed)
+    {
         return NULL;
+    }
 
     snprintf (str_reinit_color, sizeof (str_reinit_color),
               "%c",
@@ -792,7 +828,7 @@ gui_bar_window_content_get_with_filling (struct t_gui_bar_window *bar_window,
             total_items = 0;
             max_length = 1;
             max_length_screen = 1;
-            split_items = malloc(bar_window->items_count * sizeof(*split_items));
+            split_items = malloc (bar_window->items_count * sizeof (*split_items));
             for (i = 0; i < bar_window->items_count; i++)
             {
                 if (bar_window->items_subcount[i] > 0)
@@ -804,8 +840,15 @@ gui_bar_window_content_get_with_filling (struct t_gui_bar_window *bar_window,
                                                                   i, sub);
                         if (ptr_content && ptr_content[0])
                         {
-                            split_items[i][sub] = string_split (ptr_content,
-                                                                "\n", 0, 0, NULL);
+                            split_items[i][sub] = string_split (
+                                ptr_content,
+                                "\n",
+                                NULL,
+                                WEECHAT_STRING_SPLIT_STRIP_LEFT
+                                | WEECHAT_STRING_SPLIT_STRIP_RIGHT
+                                | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS,
+                                0,
+                                NULL);
                             for (j = 0; split_items[i][sub][j]; j++)
                             {
                                 total_items++;
@@ -880,6 +923,7 @@ gui_bar_window_content_get_with_filling (struct t_gui_bar_window *bar_window,
                     if (content)
                         free (content);
                     free (split_items);
+                    free (linear_items);
                     return NULL;
                 }
                 content = content2;
@@ -964,6 +1008,9 @@ gui_bar_window_coords_add (struct t_gui_bar_window *bar_window,
 {
     struct t_gui_bar_window_coords **coords2;
 
+    if (!bar_window)
+        return;
+
     if (!bar_window->coords)
     {
         bar_window->coords_count = 1;
@@ -1003,6 +1050,9 @@ gui_bar_window_coords_free (struct t_gui_bar_window *bar_window)
 {
     int i;
 
+    if (!bar_window)
+        return;
+
     if (bar_window->coords)
     {
         for (i = 0; i < bar_window->coords_count; i++)
@@ -1026,6 +1076,9 @@ gui_bar_window_insert (struct t_gui_bar_window *bar_window,
 {
     struct t_gui_bar_window *pos_bar_window;
 
+    if (!bar_window || !window)
+        return;
+
     if (window->bar_windows)
     {
         pos_bar_window = gui_bar_window_find_pos (bar_window->bar, window);
@@ -1075,7 +1128,7 @@ gui_bar_window_new (struct t_gui_bar *bar, struct t_gui_window *window)
     if (window)
     {
         if ((CONFIG_INTEGER(bar->options[GUI_BAR_OPTION_TYPE]) == GUI_BAR_TYPE_WINDOW)
-            && (!gui_bar_check_conditions_for_window (bar, window)))
+            && (!gui_bar_check_conditions (bar, window)))
             return;
     }
 
@@ -1135,7 +1188,7 @@ gui_bar_window_new (struct t_gui_bar *bar, struct t_gui_window *window)
 int
 gui_bar_window_get_current_size (struct t_gui_bar_window *bar_window)
 {
-    return bar_window->current_size;
+    return (bar_window) ? bar_window->current_size : 0;
 }
 
 /*
@@ -1214,6 +1267,9 @@ gui_bar_window_set_current_size (struct t_gui_bar_window *bar_window,
 {
     int new_size, max_size;
 
+    if (!bar_window)
+        return;
+
     if (size == 0)
         new_size = 1;
     else
@@ -1256,6 +1312,9 @@ void
 gui_bar_window_free (struct t_gui_bar_window *bar_window,
                      struct t_gui_window *window)
 {
+    if (!bar_window)
+        return;
+
     /* remove window bar from list */
     if (window)
     {
@@ -1268,6 +1327,11 @@ gui_bar_window_free (struct t_gui_bar_window *bar_window,
         if (window->last_bar_window == bar_window)
             window->last_bar_window = bar_window->prev_bar_window;
     }
+    else
+    {
+        if (bar_window->bar)
+            (bar_window->bar)->bar_window = NULL;
+    }
 
     /* free data */
     gui_bar_window_content_free (bar_window);
@@ -1281,7 +1345,10 @@ gui_bar_window_free (struct t_gui_bar_window *bar_window,
 }
 
 /*
- * Removes unused bars for a window.
+ * Removes unused bars, according to bars conditions.
+ *
+ * If window is NULL, unused root bars are removed.
+ * If window is not NULL, unused window bars in this window are removed.
  *
  * Returns:
  *   1: at least one bar was removed
@@ -1293,29 +1360,51 @@ gui_bar_window_remove_unused_bars (struct t_gui_window *window)
 {
     int rc;
     struct t_gui_bar_window *ptr_bar_win, *next_bar_win;
+    struct t_gui_bar *ptr_bar;
 
     rc = 0;
 
-    ptr_bar_win = window->bar_windows;
-    while (ptr_bar_win)
+    if (window)
     {
-        next_bar_win = ptr_bar_win->next_bar_window;
-
-        if ((CONFIG_INTEGER(ptr_bar_win->bar->options[GUI_BAR_OPTION_TYPE]) == GUI_BAR_TYPE_WINDOW)
-            && (!gui_bar_check_conditions_for_window (ptr_bar_win->bar, window)))
+        /* remove unused window bars in window */
+        ptr_bar_win = window->bar_windows;
+        while (ptr_bar_win)
         {
-            gui_bar_window_free (ptr_bar_win, window);
-            rc = 1;
-        }
+            next_bar_win = ptr_bar_win->next_bar_window;
 
-        ptr_bar_win = next_bar_win;
+            if ((CONFIG_INTEGER(ptr_bar_win->bar->options[GUI_BAR_OPTION_TYPE]) == GUI_BAR_TYPE_WINDOW)
+                && (!gui_bar_check_conditions (ptr_bar_win->bar, window)))
+            {
+                gui_bar_window_free (ptr_bar_win, window);
+                rc = 1;
+            }
+
+            ptr_bar_win = next_bar_win;
+        }
+    }
+    else
+    {
+        /* remove unused root bars */
+        for (ptr_bar = gui_bars; ptr_bar; ptr_bar = ptr_bar->next_bar)
+        {
+            if ((CONFIG_INTEGER(ptr_bar->options[GUI_BAR_OPTION_TYPE]) == GUI_BAR_TYPE_ROOT)
+                && ptr_bar->bar_window
+                && (!gui_bar_check_conditions (ptr_bar, NULL)))
+            {
+                gui_bar_window_free (ptr_bar->bar_window, NULL);
+                rc = 1;
+            }
+        }
     }
 
     return rc;
 }
 
 /*
- * Adds missing bars for a window.
+ * Adds missing bars, according to bars conditions.
+ *
+ * If window is NULL, missing root bars are added.
+ * If window is not NULL, missing window bars in this window are added.
  *
  * Returns:
  *   1: at least one bar was created
@@ -1330,18 +1419,34 @@ gui_bar_window_add_missing_bars (struct t_gui_window *window)
 
     rc = 0;
 
-    for (ptr_bar = gui_bars; ptr_bar; ptr_bar = ptr_bar->next_bar)
+    if (window)
     {
-        if ((CONFIG_INTEGER(ptr_bar->options[GUI_BAR_OPTION_TYPE]) == GUI_BAR_TYPE_WINDOW)
-            && gui_bar_check_conditions_for_window (ptr_bar, window))
+        /* add missing window bars in window */
+        for (ptr_bar = gui_bars; ptr_bar; ptr_bar = ptr_bar->next_bar)
         {
-            if (!gui_bar_window_search_bar (window, ptr_bar))
+            if ((CONFIG_INTEGER(ptr_bar->options[GUI_BAR_OPTION_TYPE]) == GUI_BAR_TYPE_WINDOW)
+                && (!gui_bar_window_search_bar (window, ptr_bar))
+                && (gui_bar_check_conditions (ptr_bar, window)))
             {
                 gui_bar_window_new (ptr_bar, window);
                 rc = 1;
             }
         }
     }
+    else
+    {
+        /* add missing root bars */
+        for (ptr_bar = gui_bars; ptr_bar; ptr_bar = ptr_bar->next_bar)
+        {
+            if ((CONFIG_INTEGER(ptr_bar->options[GUI_BAR_OPTION_TYPE]) == GUI_BAR_TYPE_ROOT)
+                && !ptr_bar->bar_window
+                && (gui_bar_check_conditions (ptr_bar, NULL)))
+            {
+                gui_bar_window_new (ptr_bar, NULL);
+                rc = 1;
+            }
+        }
+    }
 
     return rc;
 }
@@ -1362,6 +1467,9 @@ gui_bar_window_scroll (struct t_gui_bar_window *bar_window,
 {
     int old_scroll_x, old_scroll_y;
 
+    if (!bar_window)
+        return;
+
     old_scroll_x = bar_window->scroll_x;
     old_scroll_y = bar_window->scroll_y;
 
@@ -1421,19 +1529,60 @@ gui_bar_window_scroll (struct t_gui_bar_window *bar_window,
 }
 
 /*
- * Returns hdata for bar window.
+ * Callback for updating data of a bar window.
  */
 
-struct t_hdata *
-gui_bar_window_hdata_bar_window_cb (void *data, const char *hdata_name)
+int
+gui_bar_window_update_cb (void *data, struct t_hdata *hdata, void *pointer,
+                          struct t_hashtable *hashtable)
 {
-    struct t_hdata *hdata;
+    const char *value;
+    int rc;
 
     /* make C compiler happy */
     (void) data;
 
+    rc = 0;
+
+    if (hashtable_has_key (hashtable, "scroll_x"))
+    {
+        value = hashtable_get (hashtable, "scroll_x");
+        if (value)
+        {
+            hdata_set (hdata, pointer, "scroll_x", value);
+            rc++;
+        }
+    }
+
+    if (hashtable_has_key (hashtable, "scroll_y"))
+    {
+        value = hashtable_get (hashtable, "scroll_y");
+        if (value)
+        {
+            hdata_set (hdata, pointer, "scroll_y", value);
+            rc++;
+        }
+    }
+
+    return rc;
+}
+
+/*
+ * Returns hdata for bar window.
+ */
+
+struct t_hdata *
+gui_bar_window_hdata_bar_window_cb (const void *pointer, void *data,
+                                    const char *hdata_name)
+{
+    struct t_hdata *hdata;
+
+    /* make C compiler happy */
+    (void) pointer;
+    (void) data;
+
     hdata = hdata_new (NULL, hdata_name, "prev_bar_window", "next_bar_window",
-                       0, 0, NULL, NULL);
+                       0, 0, &gui_bar_window_update_cb, NULL);
     if (hdata)
     {
         HDATA_VAR(struct t_gui_bar_window, bar, POINTER, 0, NULL, "bar");
@@ -1441,8 +1590,8 @@ gui_bar_window_hdata_bar_window_cb (void *data, const char *hdata_name)
         HDATA_VAR(struct t_gui_bar_window, y, INTEGER, 0, NULL, NULL);
         HDATA_VAR(struct t_gui_bar_window, width, INTEGER, 0, NULL, NULL);
         HDATA_VAR(struct t_gui_bar_window, height, INTEGER, 0, NULL, NULL);
-        HDATA_VAR(struct t_gui_bar_window, scroll_x, INTEGER, 0, NULL, NULL);
-        HDATA_VAR(struct t_gui_bar_window, scroll_y, INTEGER, 0, NULL, NULL);
+        HDATA_VAR(struct t_gui_bar_window, scroll_x, INTEGER, 1, NULL, NULL);
+        HDATA_VAR(struct t_gui_bar_window, scroll_y, INTEGER, 1, NULL, NULL);
         HDATA_VAR(struct t_gui_bar_window, cursor_x, INTEGER, 0, NULL, NULL);
         HDATA_VAR(struct t_gui_bar_window, cursor_y, INTEGER, 0, NULL, NULL);
         HDATA_VAR(struct t_gui_bar_window, current_size, INTEGER, 0, NULL, NULL);
