@@ -1,7 +1,7 @@
 /*
  * buflist-command.c - buflist command
  *
- * Copyright (C) 2003-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2019 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <stdlib.h>
@@ -112,6 +112,10 @@ buflist_command_init ()
            "    - ${format_number}: indented number with separator "
            "(evaluation of option buflist.format.number)\n"
            "    - ${number}: indented number, for example \" 1\" if there "
+           "are between 10 and 99 buffers; for merged buffers, this variable "
+           "is set with number for the first buffer and spaces for the next "
+           "buffers with same number\n"
+           "    - ${number2}: indented number, for example \" 1\" if there "
            "are between 10 and 99 buffers\n"
            "    - ${number_displayed}: \"1\" if the number is displayed, "
            "otherwise \"0\"\n"
