@@ -1,7 +1,7 @@
 /*
  * buflist.c - Bar with list of buffers
  *
- * Copyright (C) 2003-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2019 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,18 +16,20 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <libgen.h>
 
 #include "../weechat-plugin.h"
 #include "buflist.h"
 #include "buflist-bar-item.h"
 #include "buflist-command.h"
 #include "buflist-config.h"
+#include "buflist-info.h"
 #include "buflist-mouse.h"
 
 
@@ -71,8 +73,8 @@ buflist_add_bar ()
  */
 
 void
-buflist_buffer_get_irc_pointers(struct t_gui_buffer *buffer,
-                                void **irc_server, void **irc_channel)
+buflist_buffer_get_irc_pointers (struct t_gui_buffer *buffer,
+                                 void **irc_server, void **irc_channel)
 {
     const char *ptr_server_name, *ptr_channel_name, *ptr_name;
     struct t_hdata *hdata_irc_server, *hdata_irc_channel;
@@ -363,7 +365,7 @@ int
 buflist_script_loaded_cb (const void *pointer, void *data, const char *signal,
                           const char *type_data, void *signal_data)
 {
-    int length;
+    char *base_name, *base_name2;
 
     /* make C compiler happy */
     (void) pointer;
@@ -378,9 +380,14 @@ buflist_script_loaded_cb (const void *pointer, void *data, const char *signal,
     if (!signal_data)
         return WEECHAT_RC_OK;
 
-    length = strlen (signal_data);
-    if ((length >= 10)
-        && (strncmp (signal_data + length - 10, "buffers.pl", 10) == 0))
+    base_name = basename (signal_data);
+    if (!base_name)
+        return WEECHAT_RC_OK;
+    base_name2 = strdup (base_name);
+    if (!base_name2)
+        return WEECHAT_RC_OK;
+
+    if (strcmp (base_name2, "buffers.pl") == 0)
     {
         weechat_printf (NULL,
                         _("%sbuflist: warning: the script buffers.pl is "
@@ -393,6 +400,8 @@ buflist_script_loaded_cb (const void *pointer, void *data, const char *signal,
                         weechat_prefix ("error"));
     }
 
+    free (base_name2);
+
     return WEECHAT_RC_OK;
 }
 
@@ -404,8 +413,25 @@ int
 weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
 {
     struct t_hashtable *keys;
-    char str_key[256];
     int i;
+    char str_key[256];
+    char *default_keys[][2] = {
+        { /* <f1>   */ "meta-OP",        "/bar scroll buflist * -100%" },
+        { /* <f1>   */ "meta2-11~",      "/bar scroll buflist * -100%" },
+        { /* <f2>   */ "meta-OQ",        "/bar scroll buflist * +100%" },
+        { /* <f2>   */ "meta2-12~",      "/bar scroll buflist * +100%" },
+        { /* c-<f1> */ "meta2-11^",      "/bar scroll buflist * -100%" },
+        { /* c-<f1> */ "meta2-1;5P",     "/bar scroll buflist * -100%" },
+        { /* c-<f2> */ "meta2-12^",      "/bar scroll buflist * +100%" },
+        { /* c-<f2> */ "meta2-1;5Q",     "/bar scroll buflist * +100%" },
+        { /* m-<f1> */ "meta-meta-OP",   "/bar scroll buflist * b"     },
+        { /* m-<f1> */ "meta-meta2-11~", "/bar scroll buflist * b"     },
+        { /* m-<f1> */ "meta2-1;3P",     "/bar scroll buflist * b"     },
+        { /* m-<f2> */ "meta-meta-OQ",   "/bar scroll buflist * e"     },
+        { /* m-<f2> */ "meta-meta2-12~", "/bar scroll buflist * e"     },
+        { /* m-<f2> */ "meta2-1;3Q",     "/bar scroll buflist * e"     },
+        { NULL, NULL },
+    };
 
     /* make C compiler happy */
     (void) argc;
@@ -445,14 +471,11 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
     if (keys)
     {
         /* default keys */
-        weechat_hashtable_set (keys,
-                               "meta-OP", "/bar scroll buflist * -100%");
-        weechat_hashtable_set (keys,
-                               "meta-OQ", "/bar scroll buflist * +100%");
-        weechat_hashtable_set (keys,
-                               "meta-meta-OP", "/bar scroll buflist * b");
-        weechat_hashtable_set (keys,
-                               "meta-meta-OQ", "/bar scroll buflist * e");
+        for (i = 0; default_keys[i][0]; i++)
+        {
+            weechat_hashtable_set (keys,
+                                   default_keys[i][0], default_keys[i][1]);
+        }
         weechat_key_bind ("default", keys);
 
         /* default mouse actions */
@@ -484,6 +507,8 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
     weechat_hook_signal ("perl_script_loaded",
                          &buflist_script_loaded_cb, NULL, NULL);
 
+    buflist_info_init ();
+
     return WEECHAT_RC_OK;
 }
 
