@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2019 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -14,7 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef WEECHAT_PLUGIN_IRC_NICK_H
@@ -35,8 +35,8 @@ struct t_irc_nick
     char *name;                     /* nickname                              */
     char *host;                     /* full hostname                         */
     char *prefixes;                 /* string with prefixes enabled for nick */
-    char prefix[2];                 /* current prefix (higher prefix set in  */
-                                    /* prefixes)                             */
+    char *prefix;                   /* current prefix (higher prefix set in  */
+                                    /* prefixes); string with just one char  */
     int away;                       /* 1 if nick is away                     */
     char *account;                  /* account name of the user              */
     char *realname;                 /* realname (aka gecos) of the user      */
@@ -48,8 +48,9 @@ struct t_irc_nick
 extern int irc_nick_valid (struct t_irc_channel *channel,
                            struct t_irc_nick *nick);
 extern int irc_nick_is_nick (const char *string);
-extern const char *irc_nick_find_color (const char *nickname);
-extern const char *irc_nick_find_color_name (const char *nickname);
+extern char *irc_nick_find_color (const char *nickname);
+extern char *irc_nick_find_color_name (const char *nickname);
+extern void irc_nick_set_host (struct t_irc_nick *nick, const char *host);
 extern int irc_nick_is_op (struct t_irc_server *server,
                            struct t_irc_nick *nick);
 extern int irc_nick_has_prefix_mode (struct t_irc_server *server,
