@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2019 Sébastien Helleu <flashcode@flashtux.org>
  * Copyright (C) 2012 Simon Arlott
  *
  * This file is part of WeeChat, the extensible chat client.
@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef WEECHAT_PLUGIN_IRC_SERVER_H
@@ -84,6 +84,7 @@ enum t_irc_server_option
     IRC_SERVER_OPTION_MSG_PART,             /* default part message          */
     IRC_SERVER_OPTION_MSG_QUIT,             /* default quit message          */
     IRC_SERVER_OPTION_NOTIFY,               /* notify list                   */
+    IRC_SERVER_OPTION_SPLIT_MSG_MAX_LENGTH, /* max length of messages        */
     /* number of server options */
     IRC_SERVER_NUM_OPTIONS,
 };
@@ -117,9 +118,12 @@ enum t_irc_server_option
 #define IRC_SERVER_NUM_OUTQUEUES_PRIO 2
 
 /* flags for irc_server_sendf() */
-#define IRC_SERVER_SEND_OUTQ_PRIO_HIGH   1
-#define IRC_SERVER_SEND_OUTQ_PRIO_LOW    2
-#define IRC_SERVER_SEND_RETURN_HASHTABLE 4
+#define IRC_SERVER_SEND_OUTQ_PRIO_HIGH   (1 << 0)
+#define IRC_SERVER_SEND_OUTQ_PRIO_LOW    (1 << 1)
+#define IRC_SERVER_SEND_RETURN_HASHTABLE (1 << 2)
+
+/* version strings */
+#define IRC_SERVER_VERSION_CAP "302"
 
 /* casemapping (string comparisons for nicks/channels) */
 enum t_irc_server_casemapping
@@ -188,13 +192,17 @@ struct t_irc_server
                                     /* (nick____1, nick____2, ...)           */
     char *nick;                     /* current nickname                      */
     char *nick_modes;               /* nick modes                            */
-    int cap_away_notify;            /* 1 if capability away-notify is enabled*/
-    int cap_account_notify;         /* 1 if CAP account-notify is enabled    */
-    int cap_extended_join;          /* 1 if CAP extended-join is enabled     */
+    char *host;                     /* self host (set if known)              */
+    int checking_cap_ls;            /* 1 if checking supported capabilities  */
+    struct t_hashtable *cap_ls;     /* list of supported capabilities        */
+    int checking_cap_list;          /* 1 if checking enabled capabilities    */
+    struct t_hashtable *cap_list;   /* list of enabled capabilities          */
     char *isupport;                 /* copy of message 005 (ISUPPORT)        */
     char *prefix_modes;             /* prefix modes from msg 005 (eg "ohv")  */
     char *prefix_chars;             /* prefix chars from msg 005 (eg "@%+")  */
     int nick_max_length;            /* max length of nick (from msg 005)     */
+    int user_max_length;            /* max length of user (from msg 005)     */
+    int host_max_length;            /* max length of host (from msg 005)     */
     int casemapping;                /* casemapping from msg 005              */
     char *chantypes;                /* chantypes from msg 005 (eg "&#")      */
     char *chanmodes;                /* chanmodes from msg 005                */
@@ -279,12 +287,18 @@ extern int irc_server_strcasecmp (struct t_irc_server *server,
 extern int irc_server_strncasecmp (struct t_irc_server *server,
                                    const char *string1, const char *string2,
                                    int max);
+extern char *irc_server_eval_expression (struct t_irc_server *server,
+                                         const char *string);
 extern int irc_server_sasl_enabled (struct t_irc_server *server);
 extern char *irc_server_get_name_without_port (const char *name);
 extern int irc_server_set_addresses (struct t_irc_server *server,
                                      const char *addresses);
-extern void irc_server_set_nicks (struct t_irc_server *server, const char *nicks);
-extern void irc_server_set_nick (struct t_irc_server *server, const char *nick);
+extern void irc_server_set_nicks (struct t_irc_server *server,
+                                  const char *nicks);
+extern void irc_server_set_nick (struct t_irc_server *server,
+                                 const char *nick);
+extern void irc_server_set_host (struct t_irc_server *server,
+                                 const char *host);
 extern const char *irc_server_get_alternate_nick (struct t_irc_server *server);
 extern const char *irc_server_get_isupport_value (struct t_irc_server *server,
                                                   const char *feature);
