@@ -1,7 +1,7 @@
 /*
  * relay-client.c - client functions for relay plugin
  *
- * Copyright (C) 2003-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2019 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <stdlib.h>
@@ -59,7 +59,7 @@ char *relay_client_data_type_string[] = /* strings for data types           */
 { "text", "binary" };
 
 char *relay_client_msg_type_string[] = /* prefix in raw buffer for message  */
-{ "", "[PING]\n", "[PONG]\n" };
+{ "", "[PING]\n", "[PONG]\n", "[CLOSE]\n" };
 
 struct t_relay_client *relay_clients = NULL;
 struct t_relay_client *last_relay_client = NULL;
@@ -214,13 +214,16 @@ relay_client_set_desc (struct t_relay_client *client)
         free (client->desc);
 
     snprintf (desc, sizeof (desc),
-              "%d/%s%s%s%s/%s",
+              "%d/%s%s%s%s/%s%s%s%s",
               client->id,
               (client->ssl) ? "ssl." : "",
               relay_protocol_string[client->protocol],
               (client->protocol_args) ? "." : "",
               (client->protocol_args) ? client->protocol_args : "",
-              client->address);
+              client->address,
+              (client->real_ip) ? "(" : "",
+              (client->real_ip) ? client->real_ip : "",
+              (client->real_ip) ? ")" : "");
 
     client->desc = strdup (desc);
 }
@@ -336,8 +339,11 @@ relay_client_recv_text (struct t_relay_client *client, const char *data)
 
         pos[0] = '\0';
 
-        lines = weechat_string_split (client->partial_message, "\n",
-                                      0, 0, &num_lines);
+        lines = weechat_string_split (client->partial_message, "\n", NULL,
+                                      WEECHAT_STRING_SPLIT_STRIP_LEFT
+                                      | WEECHAT_STRING_SPLIT_STRIP_RIGHT
+                                      | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS,
+                                      0, &num_lines);
         if (lines)
         {
             for (i = 0; i < num_lines; i++)
@@ -422,6 +428,10 @@ relay_client_recv_text (struct t_relay_client *client, const char *data)
                             client->http_headers, "x-real-ip");
                         if (ptr_real_ip)
                         {
+                            if (client->real_ip)
+                                free (client->real_ip);
+                            client->real_ip = strdup (ptr_real_ip);
+                            relay_client_set_desc (client);
                             weechat_printf_date_tags (
                                 NULL, 0, "relay_client",
                                 _("%s: websocket client %s%s%s has real IP "
@@ -520,6 +530,24 @@ relay_client_recv_text_buffer (struct t_relay_client *client,
                                    strlen (buffer + index + 1),
                                    NULL);
             }
+            else if (msg_type == RELAY_CLIENT_MSG_CLOSE)
+            {
+                /* print message in raw buffer */
+                relay_raw_print (client, RELAY_CLIENT_MSG_CLOSE,
+                                 RELAY_RAW_FLAG_RECV | RELAY_RAW_FLAG_BINARY,
+                                 buffer + index + 1,
+                                 strlen (buffer + index + 1));
+                /* answer with a CLOSE */
+                relay_client_send (client,
+                                   RELAY_CLIENT_MSG_CLOSE,
+                                   buffer + index + 1,
+                                   strlen (buffer + index + 1),
+                                   NULL);
+                /* close the connection */
+                relay_client_set_status (client, RELAY_STATUS_DISCONNECTED);
+                /* ignore any other message after the close */
+                return;
+            }
             index++;
         }
 
@@ -862,6 +890,7 @@ relay_client_send (struct t_relay_client *client,
         raw_size[0] = data_size;
         if ((msg_type == RELAY_CLIENT_MSG_PING)
             || (msg_type == RELAY_CLIENT_MSG_PONG)
+            || (msg_type == RELAY_CLIENT_MSG_CLOSE)
             || ((client->websocket != 1)
                 && (client->send_data_type == RELAY_CLIENT_DATA_BINARY)))
         {
@@ -890,6 +919,9 @@ relay_client_send (struct t_relay_client *client,
             case RELAY_CLIENT_MSG_PONG:
                 opcode = WEBSOCKET_FRAME_OPCODE_PONG;
                 break;
+            case RELAY_CLIENT_MSG_CLOSE:
+                opcode = WEBSOCKET_FRAME_OPCODE_CLOSE;
+                break;
             default:
                 opcode = (client->send_data_type == RELAY_CLIENT_DATA_TEXT) ?
                     WEBSOCKET_FRAME_OPCODE_TEXT : WEBSOCKET_FRAME_OPCODE_BINARY;
@@ -949,7 +981,7 @@ relay_client_send (struct t_relay_client *client,
                                            NULL, NULL, NULL, NULL);
             }
         }
-        else if (num_sent < 0)
+        else
         {
 #ifdef HAVE_GNUTLS
             if (client->ssl)
@@ -1121,7 +1153,7 @@ relay_client_timer_cb (const void *pointer, void *data, int remaining_calls)
                         break;
                     }
                 }
-                else if (num_sent < 0)
+                else
                 {
 #ifdef HAVE_GNUTLS
                     if (ptr_client->ssl)
@@ -1213,7 +1245,9 @@ relay_client_new (int sock, const char *address, struct t_relay_server *server)
 #endif /* HAVE_GNUTLS */
         new_client->websocket = 0;
         new_client->http_headers = NULL;
-        new_client->address = strdup ((address) ? address : "?");
+        new_client->address = strdup ((address && address[0]) ?
+                                      address : "local");
+        new_client->real_ip = NULL;
         new_client->status = RELAY_STATUS_CONNECTED;
         new_client->protocol = server->protocol;
         new_client->protocol_string = (server->protocol_string) ? strdup (server->protocol_string) : NULL;
@@ -1321,13 +1355,26 @@ relay_client_new (int sock, const char *address, struct t_relay_server *server)
             last_relay_client = new_client;
         relay_clients = new_client;
 
-        weechat_printf_date_tags (NULL, 0, "relay_client",
-                                  _("%s: new client on port %d: %s%s%s"),
-                                  RELAY_PLUGIN_NAME,
-                                  server->port,
-                                  RELAY_COLOR_CHAT_CLIENT,
-                                  new_client->desc,
-                                  RELAY_COLOR_CHAT);
+        if (server->unix_socket)
+        {
+            weechat_printf_date_tags (NULL, 0, "relay_client",
+                                      _("%s: new client on path %s: %s%s%s"),
+                                      RELAY_PLUGIN_NAME,
+                                      server->path,
+                                      RELAY_COLOR_CHAT_CLIENT,
+                                      new_client->desc,
+                                      RELAY_COLOR_CHAT);
+        }
+        else
+        {
+            weechat_printf_date_tags (NULL, 0, "relay_client",
+                                      _("%s: new client on port %s: %s%s%s"),
+                                      RELAY_PLUGIN_NAME,
+                                      server->path,
+                                      RELAY_COLOR_CHAT_CLIENT,
+                                      new_client->desc,
+                                      RELAY_COLOR_CHAT);
+        }
 
         new_client->hook_fd = weechat_hook_fd (new_client->sock,
                                                1, 0, 0,
@@ -1384,6 +1431,8 @@ relay_client_new_with_infolist (struct t_infolist *infolist)
         new_client->websocket = weechat_infolist_integer (infolist, "websocket");
         new_client->http_headers = NULL;
         new_client->address = strdup (weechat_infolist_string (infolist, "address"));
+        str = weechat_infolist_string (infolist, "real_ip");
+        new_client->real_ip = (str) ? strdup (str) : NULL;
         new_client->status = weechat_infolist_integer (infolist, "status");
         new_client->protocol = weechat_infolist_integer (infolist, "protocol");
         str = weechat_infolist_string (infolist, "protocol_string");
@@ -1571,6 +1620,8 @@ relay_client_free (struct t_relay_client *client)
         free (client->desc);
     if (client->address)
         free (client->address);
+    if (client->real_ip)
+        free (client->real_ip);
     if (client->protocol_string)
         free (client->protocol_string);
     if (client->protocol_args)
@@ -1697,6 +1748,8 @@ relay_client_add_to_infolist (struct t_infolist *infolist,
         return 0;
     if (!weechat_infolist_new_var_string (ptr_item, "address", client->address))
         return 0;
+    if (!weechat_infolist_new_var_string (ptr_item, "real_ip", client->real_ip))
+        return 0;
     if (!weechat_infolist_new_var_integer (ptr_item, "status", client->status))
         return 0;
     if (!weechat_infolist_new_var_string (ptr_item, "status_string", relay_client_status_string[client->status]))
@@ -1776,6 +1829,7 @@ relay_client_print_log ()
                             ptr_client->http_headers,
                             weechat_hashtable_get_string (ptr_client->http_headers, "keys_values"));
         weechat_log_printf ("  address . . . . . . . : '%s'", ptr_client->address);
+        weechat_log_printf ("  real_ip . . . . . . . : '%s'", ptr_client->real_ip);
         weechat_log_printf ("  status. . . . . . . . : %d (%s)",
                             ptr_client->status,
                             relay_client_status_string[ptr_client->status]);
