@@ -1,7 +1,7 @@
 /*
  * relay-completion.c - completion for relay command
  *
- * Copyright (C) 2003-2014 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2019 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <stdlib.h>
@@ -33,23 +33,26 @@
  */
 
 int
-relay_completion_protocol_name_cb (void *data, const char *completion_item,
+relay_completion_protocol_name_cb (const void *pointer, void *data,
+                                   const char *completion_item,
                                    struct t_gui_buffer *buffer,
                                    struct t_gui_completion *completion)
 {
     struct t_infolist *infolist;
-    char protocol_name[512];
+    char protocol_name[1024];
 
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
     (void) buffer;
     (void) completion_item;
 
-    infolist = weechat_infolist_get("irc_server", NULL, NULL);
+    infolist = weechat_infolist_get ("irc_server", NULL, NULL);
     if (infolist)
     {
         while (weechat_infolist_next (infolist))
         {
+            /* TCP socket */
             snprintf (protocol_name, sizeof (protocol_name), "irc.%s",
                       weechat_infolist_string (infolist, "name"));
             weechat_hook_completion_list_add (completion, protocol_name,
@@ -58,15 +61,31 @@ relay_completion_protocol_name_cb (void *data, const char *completion_item,
                       weechat_infolist_string (infolist, "name"));
             weechat_hook_completion_list_add (completion, protocol_name,
                                               0, WEECHAT_LIST_POS_SORT);
+            /* UNIX domain socket */
+            snprintf (protocol_name, sizeof (protocol_name), "unix.irc.%s",
+                      weechat_infolist_string (infolist, "name"));
+            weechat_hook_completion_list_add (completion, protocol_name,
+                                              0, WEECHAT_LIST_POS_SORT);
+            snprintf (protocol_name, sizeof (protocol_name), "unix.ssl.irc.%s",
+                      weechat_infolist_string (infolist, "name"));
+            weechat_hook_completion_list_add (completion, protocol_name,
+                                              0, WEECHAT_LIST_POS_SORT);
         }
         weechat_infolist_free (infolist);
     }
 
+    /* TCP socket */
     weechat_hook_completion_list_add (completion, "weechat",
                                       0, WEECHAT_LIST_POS_SORT);
     weechat_hook_completion_list_add (completion, "ssl.weechat",
                                       0, WEECHAT_LIST_POS_SORT);
 
+    /* UNIX domain socket */
+    weechat_hook_completion_list_add (completion, "unix.weechat",
+                                      0, WEECHAT_LIST_POS_SORT);
+    weechat_hook_completion_list_add (completion, "unix.ssl.weechat",
+                                      0, WEECHAT_LIST_POS_SORT);
+
     return WEECHAT_RC_OK;
 }
 
@@ -75,13 +94,15 @@ relay_completion_protocol_name_cb (void *data, const char *completion_item,
  */
 
 int
-relay_completion_relays_cb (void *data, const char *completion_item,
+relay_completion_relays_cb (const void *pointer, void *data,
+                            const char *completion_item,
                             struct t_gui_buffer *buffer,
                             struct t_gui_completion *completion)
 {
     struct t_relay_server *ptr_server;
 
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
     (void) buffer;
     (void) completion_item;
@@ -102,7 +123,8 @@ relay_completion_relays_cb (void *data, const char *completion_item,
  */
 
 int
-relay_completion_free_port_cb (void *data, const char *completion_item,
+relay_completion_free_port_cb (const void *pointer, void *data,
+                               const char *completion_item,
                                struct t_gui_buffer *buffer,
                                struct t_gui_completion *completion)
 {
@@ -111,6 +133,7 @@ relay_completion_free_port_cb (void *data, const char *completion_item,
     int port_max;
 
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
     (void) buffer;
     (void) completion_item;
@@ -141,12 +164,12 @@ relay_completion_init ()
 {
     weechat_hook_completion ("relay_protocol_name",
                              N_("all possible protocol.name for relay plugin"),
-                             &relay_completion_protocol_name_cb, NULL);
+                             &relay_completion_protocol_name_cb, NULL, NULL);
     weechat_hook_completion ("relay_relays",
                              N_("protocol.name of current relays for relay "
                                 "plugin"),
-                             &relay_completion_relays_cb, NULL);
+                             &relay_completion_relays_cb, NULL, NULL);
     weechat_hook_completion ("relay_free_port",
                              N_("first free port for relay plugin"),
-                             &relay_completion_free_port_cb, NULL);
+                             &relay_completion_free_port_cb, NULL, NULL);
 }
