@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2019 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -14,7 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef WEECHAT_PLUGIN_SCRIPT_REPO_H
@@ -23,11 +23,11 @@
 #include <time.h>
 
 /* status for script */
-#define SCRIPT_STATUS_INSTALLED   1
-#define SCRIPT_STATUS_AUTOLOADED  2
-#define SCRIPT_STATUS_HELD        4
-#define SCRIPT_STATUS_RUNNING     8
-#define SCRIPT_STATUS_NEW_VERSION 16
+#define SCRIPT_STATUS_INSTALLED   (1 << 0)
+#define SCRIPT_STATUS_AUTOLOADED  (1 << 1)
+#define SCRIPT_STATUS_HELD        (1 << 2)
+#define SCRIPT_STATUS_RUNNING     (1 << 3)
+#define SCRIPT_STATUS_NEW_VERSION (1 << 4)
 
 struct t_script_repo
 {
@@ -43,7 +43,7 @@ struct t_script_repo
     char *requirements;                  /* requirements                    */
     char *min_weechat;                   /* min WeeChat version             */
     char *max_weechat;                   /* max WeeChat version             */
-    char *md5sum;                        /* md5sum of script content        */
+    char *sha512sum;                     /* SHA-512 checksum of script      */
     char *url;                           /* URL to download script          */
     int popularity;                      /* >0 for popular scripts only     */
     time_t date_added;                   /* date added                      */
